﻿using System;
using System.Configuration;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using System.Xml.XPath;
using System.Globalization;
using XMailAdminProxy;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;
using System.Web.SessionState;
using MailBee.Pop3Mail;
using MailBee;

/// <summary>
/// Summary description for Utils
/// </summary>
public struct SelectedValueStruct
{
    public string Type;
    public string Name;
}

public struct PluginInfo
{
    public int ID;
    public string Caption;
	public string FolderName;
}

[Flags]
public enum OutputDomains
{
	None = 0,
	XMailDomains = 1,
	WebMailDomains = 2
}


public class Utils
{

    public static bool ValidationLicenseKey()
    {
        AdminPanelSettings apSettings = new AdminPanelSettings();
        apSettings.LoadAdminPanelSettings();

        bool flag = false;
        try
        {
            Pop3.LicenseKey = apSettings.LicenseKey;
            Pop3 pop = new Pop3();
            flag = true;
        }
        catch (MailBeeException)
        {
            flag = false;
        }
        return flag;
    }
    
    public static List<PluginInfo> GetPluginInfos(string rootFolder)
	{
        List<PluginInfo> result = new List<PluginInfo>();

        bool flag = false;
        
        if (Utils.IsPluginExist("WebMail"))
        {
            flag = Utils.ValidationLicenseKey();
        }
        else if (Utils.IsPluginExist("WebMailLite"))
        {
            flag = true;   
        }
        
        if (flag)
        {
            try
            {
                string settingsFile = Path.Combine(Utils.GetAdminPanelDataFolderPath(), "adminpanel.xml");
                if (File.Exists(settingsFile))
                {
                    XmlDocument xmlDoc = new XmlDocument();
                    xmlDoc.Load(settingsFile);

                    XmlNodeReader nodeReader = new XmlNodeReader(xmlDoc);

                    XPathDocument xpathDoc = new XPathDocument(nodeReader);
                    XPathNavigator xpathNav = xpathDoc.CreateNavigator();

                    XPathNodeIterator xpathParamNodeIter = xpathNav.Select(@"settings/plugins/plugin");
                    const int Num = 0;
                    PluginInfo pInfo = new PluginInfo();
                    while (xpathParamNodeIter.MoveNext())
                    {
                        string caption = xpathParamNodeIter.Current.GetAttribute("caption", "");
                        string folderName = xpathParamNodeIter.Current.GetAttribute("folderName", "");
                        if ((caption == null) || (folderName == null)) continue;
                        if (Directory.Exists(Path.Combine(rootFolder, "PlugIns/" + folderName)))
                        {
                            pInfo.ID = Num;
                            pInfo.Caption = caption;
                            pInfo.FolderName = folderName;
                            result.Add(pInfo);
                        }
                    }
                }
            }
            catch (Exception)
            {
                throw;
            }
        }
        else
        {
            PluginInfo pi = Utils.GetPluginInfo("LicenseKey", rootFolder);
            result.Add(pi);
        }

        return result;
	}

    public static PluginInfo GetPluginInfo(string plugin, string rootFolder)
    {
        PluginInfo result = new PluginInfo();
        try
        {
            string settingsFile = Path.Combine(GetAdminPanelDataFolderPath(), "adminpanel.xml");
            if (File.Exists(settingsFile))
            {
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(settingsFile);

                XmlNodeReader nodeReader = new XmlNodeReader(xmlDoc);

                XPathDocument xpathDoc = new XPathDocument(nodeReader);
                XPathNavigator xpathNav = xpathDoc.CreateNavigator();

                XPathNodeIterator xpathParamNodeIter = xpathNav.Select(@"settings/plugins/plugin");
                int Num = 0;
                while (xpathParamNodeIter.MoveNext())
                {
                    string caption = xpathParamNodeIter.Current.GetAttribute("caption", "");
                    string folderName = xpathParamNodeIter.Current.GetAttribute("folderName", "");
                    if (folderName == plugin)
                    {
                        if (Directory.Exists(Path.Combine(rootFolder, "PlugIns/" + folderName)))
                        {
                            result.ID = Num;
                            result.Caption = caption;
                            result.FolderName = folderName;
                        }
                    }
                    Num++;
                }
            }
        }
        catch (Exception)
        {
            throw;
        }
        return result;
    }

    public static bool IsPluginExist(string PlugName)
    {
        string settingsFile = Path.Combine(GetAdminPanelDataFolderPath(), "adminpanel.xml");
        try
        {
            if (File.Exists(settingsFile))
            {
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(settingsFile);

                XmlNodeReader nodeReader = new XmlNodeReader(xmlDoc);

                XPathDocument xpathDoc = new XPathDocument(nodeReader);
                XPathNavigator xpathNav = xpathDoc.CreateNavigator();

                XPathNodeIterator xpathParamNodeIter = xpathNav.Select(@"settings/plugins/plugin");
                while (xpathParamNodeIter.MoveNext())
                {
                    string folderName = xpathParamNodeIter.Current.GetAttribute("folderName", "");

                    if ((folderName == PlugName)/* && Directory.Exists(Directory.GetCurrentDirectory() + @"/PlugIns/" + folderName)*/)
                    {
                        return true;
                    }
                }
            }
        }
        catch (Exception)
        {
            throw;
        }
        return false;
    }

	public static string GetWebMailFolder()
	{
	    string strDataFolderPath = "";

        strDataFolderPath = ConfigurationManager.AppSettings["WebMailFolderPath"];	        
        if (HttpContext.Current.Request.PhysicalApplicationPath != null)
        {
            Uri appUri = new Uri(HttpContext.Current.Request.PhysicalApplicationPath);
            Uri newUri = new Uri(appUri, ConfigurationManager.AppSettings["WebMailFolderPath"]);
            strDataFolderPath = newUri.LocalPath;
        }

        return strDataFolderPath;
    }

    public static string GetAdminPanelDataFolderPath()
    {
        string result = "";
        if (ConfigurationManager.AppSettings["AdminPanelDataFolderPath"] != null)
        {
            result = ConfigurationManager.AppSettings["AdminPanelDataFolderPath"];
        }
        else
        {
            try
            {
                result = Path.Combine(GetWebMailDataFolder(), "settings");
            }
            catch (Exception)
            {
                result = "";
            }
        }
        return result;
    }

    public static string GetWebMailDataFolder()
	{
        try
        {
            string webmailFolder = GetWebMailFolder();
            if (webmailFolder != null)
            {
                string webconfigPath = Path.Combine(webmailFolder, "web.config");
                if (File.Exists(webconfigPath))
                {
                    XmlDocument xmlDoc = new XmlDocument();
                    xmlDoc.Load(webconfigPath);

                    XmlNodeReader nodeReader = new XmlNodeReader(xmlDoc);

                    XPathDocument xpathDoc = new XPathDocument(nodeReader);
                    XPathNavigator xpathNav = xpathDoc.CreateNavigator();

                    XPathNodeIterator xpathParamNodeIter = xpathNav.Select(@"configuration/appSettings/add");
                    while (xpathParamNodeIter.MoveNext())
                    {
                        string key = xpathParamNodeIter.Current.GetAttribute("key", "");
                        string value = xpathParamNodeIter.Current.GetAttribute("value", "");
                        if ((key != null) && (value != null))
                        {
                            if (string.Compare(key, "dataFolderPath", true, CultureInfo.InvariantCulture) == 0)
                            {
                                if ((HttpContext.Current != null) && (HttpContext.Current.Application != null))
                                {
                                    if (HttpContext.Current.Application[WebMailPro.Constants.appSettingsDataFolderPath] != null)
                                        HttpContext.Current.Application.Add(WebMailPro.Constants.appSettingsDataFolderPath, value);
                                    else
                                        HttpContext.Current.Application[WebMailPro.Constants.appSettingsDataFolderPath] = value;
                                }
                                Uri appUri = new Uri(webmailFolder);
                                Uri newUri = new Uri(appUri, value);
                                return newUri.LocalPath;
                            }
                        }
                    }
                }
            }
        }
        catch (Exception)
        {
            throw;
        }
        return null;
	}

	public static void SaveState(string key, object value, HttpSessionState session)
	{
        session[key] = value;
	}

    public static object LoadState(string key, HttpSessionState session)
	{
		return session[key];
	}

    public static XMailAdminProxy.XMLayer CreateXMLayer()
    {
        AdminPanelSettings apSettings = new AdminPanelSettings();
        apSettings.LoadAdminPanelSettings();
        return CreateXMLayer(apSettings.User, apSettings.Pass, apSettings.Host, apSettings.Port);
    }

    public static XMailAdminProxy.XMLayer CreateXMLayer(string Login, string Password, string Host, string Port)
    {
        return new XMLayer(Login, Password, Host, Convert.ToInt32(Port));
    }

    public static string[] GetSupportedSkins(string webmailFolderPath)
	{
		if (webmailFolderPath == null) return null;
		string skinsFolder = Path.Combine(webmailFolderPath, "skins");
		ArrayList skinsArr = new ArrayList();
		if (Directory.Exists(skinsFolder))
		{
			string[] skins = Directory.GetDirectories(skinsFolder);
			foreach (string skin in skins)
			{
				skinsArr.Add(Path.GetFileName(skin));
			}
		}
		return (string[])skinsArr.ToArray(typeof(string));
	}

	public static string[] GetSupportedLangs(string dataFolderPath)
	{
		ArrayList arr = new ArrayList();
		string langsXml = Path.Combine(dataFolderPath, @"langs\langs.xml");
		if (File.Exists(langsXml))
		{
			XPathDocument xpathDoc = new XPathDocument(langsXml);
			XPathNavigator nav = xpathDoc.CreateNavigator();
			XPathNodeIterator langIter = nav.Select(string.Format("langs/lang/FriendlyName"));
			while (langIter.MoveNext())
			{
				arr.Add(langIter.Current.Value);
			}
		}
		return (arr.Count > 0) ? (string[])arr.ToArray(typeof(string)) : new string[] { WebMailPro.Constants.defaultLang };
	}

    public static string EncryptPassword(string password)
    {
        StringBuilder sb = new StringBuilder();
        byte[] bytes = Encoding.UTF8.GetBytes(password);
        for (int i = 0; i < bytes.Length; i++)
        {
            byte b = (byte)((bytes[i] ^ 101) & 0xFF);
            sb.Append(b.ToString("X2"));
        }
        return sb.ToString();
    }

    public static string DecryptPassword(string password)
    {
        if (password == null) return string.Empty;
        string result = string.Empty;
        if ((password.Length > 0) && (password.Length % 2 == 0))
        {
            byte[] decryptedBytes = new byte[password.Length / 2];
            int startIndex = 0;
            int index = 0;
            while (startIndex < password.Length)
            {
                string strByte = password.Substring(startIndex, 2);
                int b = 0;
                if (int.TryParse(strByte, NumberStyles.HexNumber, CultureInfo.InvariantCulture, out b))
                {
                    b = ((b & 0xFF) ^ 101);
                    decryptedBytes[index] = (byte)b;
                }
                startIndex += 2;
                index++;
            }
            result = Encoding.UTF8.GetString(decryptedBytes);
        }
        return result;
    }

    public static string EncodeHtml(string s)
    {
        Regex rChar = new Regex("[\x0-\x8\xB-\xC\xE-\x1F]+");
        s = rChar.Replace(s, " ");

        StringBuilder sb = new StringBuilder(s);
        sb.Replace("&", "&amp;");
        sb.Replace("<", "&lt;");
        sb.Replace(">", "&gt;");
        return sb.ToString();
    }

    public static string EncodeHtmlSimple(string s)
    {
        StringBuilder sb = new StringBuilder(s);
        sb.Replace("<", "&lt;");
        sb.Replace(">", "&gt;");
        return sb.ToString();
    }

    public static string DecodeHtml(string s)
    {
        StringBuilder sb = new StringBuilder(s);
        sb.Replace("&lt;", "<");
        sb.Replace("&gt;", ">");
        sb.Replace("&amp;", "&");
        return sb.ToString();
    }

    public static void SetPageErrorMessage(Page page, string err)
    {
        HiddenField hfErrorMessage = (HiddenField)page.FindControl("errorMessage");
        hfErrorMessage.Value = err;
    }
    public static void SetPageReportMessage(Page page, string err)
    {
        HiddenField hfReportMessage = (HiddenField)page.FindControl("reportMessage");
        hfReportMessage.Value = err;
    }

    public static SelectedValueStruct ParseSelectedValue(string Domain)
    {
        SelectedValueStruct sds = new SelectedValueStruct();
        if (Domain.Trim() != "")
        {
            string[] domains = Domain.Split(new Char[] {'_'}, 2);
            if (domains.Length > 1)
            {
                sds.Type = domains[0];
                sds.Name = domains[1];
            }
        }
        return sds;
    }
}
