using System;
using System.Collections;
using System.Globalization;
using System.IO;
using System.Text.RegularExpressions;
using System.Web.UI;
using MailBee.Html;
using MailBee.ImapMail;
using MailBee.Mime;

namespace WebMailLite
{
	/// <summary>
	/// Summary description for BaseWebMailActions.
	/// </summary>
	public class BaseWebMailActions
	{
		protected Account _acct = null;
		protected Page _webPage = null;

		public Account CurrentAccount
		{
			get { return _acct; }
		}

		public Page CurrentPage
		{
			get { return _webPage; }
		}

		public BaseWebMailActions(Account acct, Page page)
		{
			_acct = acct;
			_webPage = page;
		}

		public void DeleteMessages(string[] uids)
		{
			MailStorage storage = MailServerStorageCreator.CreateMailServerStorage(_acct);
			try
			{
				storage.Connect(false);
				storage.DeleteMessages(uids);
			}
			finally
			{
				storage.Disconnect();
			}
		}

		public MailMessage GetMessage(string uid, int charset)
		{
            byte safety = 0;
			return GetMessage(uid, charset, safety);
		}


		public MailMessage GetMessage(string uid, int charset, byte safety)
		{
			MailMessage msg = null;
			if (_acct != null)
			{
				MailStorage storage = MailServerStorageCreator.CreateMailServerStorage(_acct);
				try
				{
					if (!storage.IsConnected()) storage.Connect(false);
					msg = storage.LoadMessage(uid);
					msg.Parser.EncodingOverride = (charset > 0) ? Utils.GetEncodingByCodePage(charset) : null;
				}
				finally
				{
					storage.Disconnect();
				}
			}
			else
			{
				if (_acct == null)
					Log.WriteLine("GetMessage", "Account is null.");
				throw new WebMailSessionException((new WebmailResourceManagerCreator()).CreateResourceManager().GetString("SessionIsEmpty"));
			}

			if (msg != null)
			{
				if (msg.Attachments.Count > 0)
				{
					if (_webPage != null)
					{
						string filename = string.Empty;
						string tempFolder = Utils.GetTempFolderName(_webPage.Session);
						msg.Parser.WorkingFolder = tempFolder;
						msg.BodyHtmlText = msg.GetHtmlAndSaveRelatedFiles(@"get_attachment_binary.aspx?filename=", VirtualMappingType.Static, MessageFolderBehavior.DoNotCreate);
						for (int i = 0; i < msg.Attachments.Count; i++)
						{
							filename = Utils.CreateTempFilePath(tempFolder,
								(msg.Attachments[i].Filename.Length > 0) ? msg.Attachments[i].Filename : msg.Attachments[i].Name);
							msg.Attachments[i].Save(filename, true);
						}
					}
				}

				if ((msg.BodyHtmlText != null) && (msg.BodyHtmlText.Length > 0))
				{
					string htmlBody = msg.BodyHtmlText;
					// MailBee.Html
					Processor pr = new Processor();
					// MailBee.Html
					pr.Dom.OuterHtml = htmlBody;
					RuleSet rs = new RuleSet();

					string[] removeAttributes = new string[] { "onActivate", "onAfterPrint",
								"onBeforePrint", "onAfterUpdate", "onBeforeUpdate", "onErrorUpdate",
								"onAbort", "onBeforeDeactivate", "onDeactivate", "onBeforeCopy",
								"onBeforeCut", "onBeforeEditFocus", "onBeforePaste", "onBeforeUnload",
								"onBlur", "onBounce", "onChange", "onClick", "onControlSelect",
								"onCopy", "onCut", "onDblClick", "onDrag", "onDragEnter", "onDragLeave",
								"onDragOver", "onDragStart", "onDrop", "onFilterChange", "onDragDrop",
								"onError", "onFilterChange", "onFinish", "onFocus", "onHelp", "onKeyDown",
								"onKeyPress", "onKeyUp", "onLoad", "onLoseCapture", "onMouseDown",
								"onMouseEnter", "onMouseLeave", "onMouseMove", "onMouseOut",
								"onMouseOver", "onMouseUp", "onMove", "onPaste",
								"onPropertyChange", "onReadyStateChange", "onReset", "onResize",
								"onResizeEnd", "onResizeStart", "onScroll", "onSelectStart",
								"onSelect", "onSelectionChange", "onStart", "onStop", "onSubmit",
								"onUnload"};

					TagAttributeCollection attrsToRemove = new TagAttributeCollection();
					foreach (string removeAttribute in removeAttributes)
					{
						TagAttribute remAttr =  new TagAttribute();
						remAttr.Name = removeAttribute;
						attrsToRemove.Add(remAttr);
					}
					rs.AddTagProcessingRule(".*", null, null, attrsToRemove, false);
					
					rs.AddTagRemovalRule("head",null);
					rs.AddTagRemovalRule("title", null);
					rs.AddTagRemovalRule("style", null);
					rs.AddTagRemovalRule("script", null);
					rs.AddTagRemovalRule("object", null);
					rs.AddTagRemovalRule("embed", null);
					rs.AddTagRemovalRule("applet", null);
					rs.AddTagRemovalRule("mocha", null);
					rs.AddTagRemovalRule("base", null);

					TagAttributeCollection attrsToAdd = new TagAttributeCollection();
					MailBee.Html.TagAttribute addAttr = new MailBee.Html.TagAttribute();
					addAttr.Name = "target";
					addAttr.Value = "\"_blank\"";
					attrsToAdd.Add(addAttr);
					rs.AddTagProcessingRule("a", null, attrsToAdd, null, false);
						
					if (safety == 0)
					{
						safety = 1; // sender not safety, but HTML may be safety
						//-------------------------------------------------------------
						//By ZaKon: background/background-image replacement
						//-------------------------------------------------------------
						#region BGImagesReplacement
						ElementReadOnlyCollection els = pr.Dom.GetAllElements();
						foreach (Element el in els)
						{
							if (el.Attributes!=null)
							{
								foreach (MailBee.Html.TagAttribute att in el.Attributes)
								{
									if (att.Name.ToLower()=="style")
									{
										Regex re = new Regex(@"background",RegexOptions.IgnoreCase);
										att.Value = re.Replace(att.Value,"wmx_background");
									}
									if (att.Name.ToLower()=="background")
									{
										att.Name = att.Name.Insert(0,"wmx_");
									}
								}
							}
						}
						#endregion
						//-------------------------------------------------------------
						ElementCollection images = pr.Images;
						foreach (Element imageElement in images)
						{
							TagAttributeCollection searchAttrs = new TagAttributeCollection();
							TagAttributeCollection addAttrs = new TagAttributeCollection();
							TagAttributeCollection removeAttrs = new TagAttributeCollection();

							foreach (MailBee.Html.TagAttribute attr in imageElement.Attributes)
							{
								if ((attr.Name != null) && (attr.Value != null))
								{
									string attrValue = attr.Value.Trim(new char[] {'\'', '"' });
									if (attr.Name.ToLower() == "src")
									{
										if (!attrValue.StartsWith("\"get_attachment_binary.aspx"))
										{
											safety = 0; // unfortunately, HTML not safety
											// external images
											MailBee.Html.TagAttribute searchAttr = new TagAttribute();
											searchAttr.Name = attr.Name;
											searchAttr.Value = Regex.Escape(attr.Value);

											searchAttrs.Add(searchAttr);

											MailBee.Html.TagAttribute oldAttr = new TagAttribute();
											oldAttr.Definition = attr.Name;
											removeAttrs.Add(oldAttr);

											MailBee.Html.TagAttribute newAttr = new TagAttribute();
											//newAttr.Definition = "src='about:blank'";
											newAttr.Name = attr.Name;
											newAttr.Value = attr.Value;
											addAttrs.Add(newAttr);

											break;
										}
									}
								}
							}
							rs.AddTagProcessingRule("img", searchAttrs, addAttrs, searchAttrs, true);
						}
					}

					string bodyHtmlText = pr.Dom.ProcessToString(rs, null);
					Regex locationRegex = new Regex(@"(document|window).location[^=]*=\s*[""'][^""']*[""']", RegexOptions.IgnoreCase);
					bodyHtmlText = locationRegex.Replace(bodyHtmlText, "");

					msg.BodyHtmlText = bodyHtmlText;
				}

				if (msg != null)
				{
                    if ((msg.BodyPlainText == null) || (msg.BodyPlainText.Length == 0))
                    {
                        msg.MakePlainBodyFromHtmlBody();
                    }
                }
			}
			else
			{
				throw new WebMailException("Message not exists.");
			}
			return msg;
		}


		public MailMessageCollection GetMessages(int page, out int inboxMessageCount, out long inboxSize)
		{
			if (_acct != null)
			{
				MailStorage storage = MailServerStorageCreator.CreateMailServerStorage(_acct);
				try
				{
					if (!storage.IsConnected()) storage.Connect(false);
					inboxMessageCount = storage.GetMessageCount();
					inboxSize = storage.GetMailStorageSize();
					return storage.LoadMessageHeaders(page);
				}
				finally
				{
					storage.Disconnect();
				}
			}
			else
			{
				if (_acct == null)
					Log.WriteLine("GetMessages", "Account is null.");
				throw new WebMailSessionException((new WebmailResourceManagerCreator()).CreateResourceManager().GetString("SessionIsEmpty"));
			}
		}

		public Account LoginAccount(string email, string login, string password, string inc_server, int inc_port, string out_server, int out_port, bool smtp_auth, bool advanced_login)
		{
			try
			{
				_acct = Account.LoginAccount(email, login, password, inc_server, inc_port, out_server, out_port, smtp_auth, advanced_login);
				return _acct;
			}
			catch (WebMailException)
			{
				throw;
			}
		}

		public void SendMessage(MailMessage msg)
		{
			if (_acct != null)
			{
                Smtp.SendMail(_acct, msg);
			}
			else
			{
				if (_acct == null)
					Log.WriteLine("NewMessage", "Account is null.");
				throw new WebMailSessionException((new WebmailResourceManagerCreator()).CreateResourceManager().GetString("SessionIsEmpty"));
			}
		}
	}
}
